-------------------------------------------------------------
--	Filename:  COM1100.VHD
-- Author: Alain Zarembowitch / MSS
--	Version: 2
--	Date last modified: 6-15-04
-- Inheritance: 	none
--
-- description:  COM-1100 module, VHDL demonstration code.
-- Includes a basic interface for the USB 2.0 physical interface,
-- two interfaces from the USB20 module for two virtual channels,
-- in which one of the channels communicates with the microcontroller.
-- Be sure to include the constraint file COM1100.UCF in the project implementation.
---------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity COM1100 is
    port ( 
		--GLOBAL CLOCKS
	   CLK_IN1 : in std_logic;	-- master clock for this FPGA, internal to the module
	   CLK_IN2 : in std_logic;	-- master clock for this FPGA, external to the module

		--// Left connector J3, 40 pins. 
		-- Pin 1 is reserved for input clock CLK_IN2
		-- Pins 10,20,30,40 reserved for Ground
		-- Pins 37,38,39 reserved for JTAG
		-- Pins 35-36 reserved for ComBlock Monitoring and Control.
		J3: inout std_logic_vector(34 downto 2);

		--// Right and Bottom connectors: J5 (right) and J1 (bottom), 40 pins. 
		-- Pin 1 is reserved to forward clock to following module.
		-- Pins 10,20,30,40 reserved for Ground
		-- Pins 37,38,39 reserved for JTAG
		-- Pins 35-36 reserved for ComBlock Monitoring and Control.
		J1: inout std_logic_vector(34 downto 1);
		J5: inout std_logic_vector(34 downto 1);

		--// Monitoring & Control: Atmel controller interface
		UC_AD: inout std_logic_vector(7 downto 0);
		UC_WRN_IN: in std_logic;
		UC_RDN: in std_logic;
		UC_ALE_IN: in std_logic;
		UC_FLASH_CSN: in std_logic;
			-- '0' when uC talks to the flash memory, '1' when otherwise (i.e. possibly
			-- talk to the FPGAs).

		--// Monitoring & Control: Serial ports interface
		RX_L: in std_logic;
		RX_B: in std_logic;
		RX_R: in std_logic;
		TX_L: out std_logic;
		TX_B: out std_logic;
		TX_R: out std_logic;

		--// USB interface
		USB_RESET: out std_logic;
		USB_DATABUS16_8: out std_logic;
		USB_SUSPENDN: out std_logic;
		USB_XCVRSELECT: out std_logic;
		USB_TERMSELECT: out std_logic;
		USB_OPMODE: out std_logic_vector(1 downto 0);
		USB_LINESTATE: in std_logic_vector(1 downto 0);
		USB_CLKOUT: in std_logic;
		USB_TXVALID: out std_logic;
		USB_TXREADY: in std_logic;
		USB_RXVALID: in std_logic;
		USB_RXACTIVE: in std_logic;
		USB_RXERROR: in std_logic;
		USB_DATA_IN: in std_logic_vector(7 downto 0);	
		USB_DATA_OUT: out std_logic_vector(7 downto 0);	
		USB_VBUS_SENSE: in std_logic;
			
		--// Test Points
		TEST_POINTS: out std_logic_vector(10 downto 1);
		INITB: out std_logic

			);
end entity;

architecture behavioral of COM1100 is
--------------------------------------------------------
--      COMPONENTS
--------------------------------------------------------
component IBUFG
      port (I : in STD_LOGIC; O : out std_logic);
end component;

component BUFG
      port (I : in STD_LOGIC; O : out std_logic);
end component;

component DCM
    port ( CLKIN     : in  std_logic;
           CLKFB     : in  std_logic;
           DSSEN     : in  std_logic;
           PSINCDEC  : in  std_logic;
           PSEN      : in  std_logic;
           PSCLK     : in  std_logic;
           RST       : in  std_logic;
           CLK0      : out std_logic;
           CLK90     : out std_logic;
           CLK180    : out std_logic;
           CLK270    : out std_logic;
           CLK2X     : out std_logic;
           CLK2X180  : out std_logic;
           CLKDV     : out std_logic;
           CLKFX     : out std_logic;
           CLKFX180  : out std_logic;
           LOCKED    : out std_logic;
           PSDONE    : out std_logic;
           STATUS    : out std_logic_vector(7 downto 0)
          );
end component;

component USB20 is
    port ( 
	 	--// Clocks Resets
		ASYNC_RESET: in std_logic;
		USB_CLK60G: in std_logic;	-- reference clock. 60 MHz
		CLK_P: in std_logic;

		--// USB interface
		USB_RESET: out std_logic;
		USB_DATABUS16_8: out std_logic;
		USB_SUSPENDN: out std_logic;
		USB_XCVRSELECT: out std_logic;
		USB_TERMSELECT: out std_logic;
		USB_OPMODE: out std_logic_vector(1 downto 0);
			-- operational mode
			-- 00 = normal operation
			-- 01 = non-driving (all terminations removed)
			-- 10 = disable bit stuffing and NRZI encoding (unused)
			-- 11 = reserved (unused)
		USB_LINESTATE: in std_logic_vector(1 downto 0);

		USB_TXVALID: out std_logic;
		USB_TXREADY: in std_logic;
		USB_RXVALID: in std_logic;
		USB_RXACTIVE: in std_logic;
		USB_RXERROR: in std_logic;

		USB_DATA_IN: in std_logic_vector(7 downto 0);	
		USB_DATA_OUT: out std_logic_vector(7 downto 0);	
		
		USB_VBUS_SENSE: in std_logic;

		-- // Data Stream 1: UC interface		
		----- the output: uC reads from the dual-port RAM with clock CLK_P
		DATA1_OUT: out std_logic_vector(7 downto 0);
		DATA1_OUT_SAMPLE_CLK: out std_logic;
			-- read DATA1_OUT at rising edge of CLK_P when DATA1_OUT_SAMPLE_CLK = '1'
		DATA1_OUT_BUFFER_EMPTY: out std_logic;
		DATA1_OUT_SAMPLE_CLK_REQ: in std_logic;	
			-- requests data. If no data is available in the buffer, the
			-- DATA1_OUT_SAMPLE_CLK will stay low.
			-- (flow control)
		
		----- the input: uC writes to the dual-port RAM with clock uC_WRN
		
		DATA1_IN: in std_logic_vector(7 downto 0);
		DATA1_IN_SAMPLE_CLK: in std_logic;
			-- read DATA1_IN at rising edge of CLK_P when DATA1_IN_SAMPLE_CLK = '1'
		DATA1_IN_SAMPLE_CLK_REQ: out std_logic;
			-- requests data when the input elastic buffer is less than half full. 
			-- (flow control)

		--// Data Stream 2
		-- Synchronous with CLK_P clock
		DATA2_OUT: out std_logic_vector(7 downto 0);
		DATA2_OUT_SAMPLE_CLK: out std_logic;
			-- read DATA2_OUT at rising edge of CLK_P when DATA2_OUT_SAMPLE_CLK = '1'
		DATA2_OUT_BUFFER_EMPTY: out std_logic;
		DATA2_OUT_SAMPLE_CLK_REQ: in std_logic;	
			-- requests data. If no data is available in the buffer, the
			-- DATA2_OUT_SAMPLE_CLK will stay low.
			-- (flow control)
		
		DATA2_IN: in std_logic_vector(7 downto 0);
		DATA2_IN_SAMPLE_CLK: in std_logic;
			-- read DATA2_IN at rising edge of CLK_P when DATA2_IN_SAMPLE_CLK = '1'
		DATA2_IN_SAMPLE_CLK_REQ: out std_logic;
			-- requests data when the input elastic buffer is less than half full. 
			-- (flow control)

		--// Test Points
		USB_TP: out std_logic_vector(10 downto 1)
);
end component;

component RAMB16_S9_S9
  port (DIA    : in STD_LOGIC_VECTOR (7 downto 0);
        DIB    : in STD_LOGIC_VECTOR (7 downto 0);
        DIPA    : in STD_LOGIC_VECTOR (0 downto 0);
        DIPB    : in STD_LOGIC_VECTOR (0 downto 0);
        ENA    : in STD_logic;
        ENB    : in STD_logic;
        WEA    : in STD_logic;
        WEB    : in STD_logic;
        SSRA   : in STD_logic;
        SSRB   : in STD_logic;
        CLKA   : in STD_logic;
        CLKB   : in STD_logic;
        ADDRA  : in STD_LOGIC_VECTOR (10 downto 0);
        ADDRB  : in STD_LOGIC_VECTOR (10 downto 0);
        DOA    : out STD_LOGIC_VECTOR (7 downto 0);
        DOB    : out STD_LOGIC_VECTOR (7 downto 0);
        DOPA    : out STD_LOGIC_VECTOR (0 downto 0);
        DOPB    : out STD_LOGIC_VECTOR (0 downto 0)
       ); 

end component;

component COMSCOPE is 
	port ( 
	--GLOBAL CLOCKS
   CLK : in std_logic;				-- Master clock for this FPGA
	ASYNC_RESET: in std_logic;		-- Asynchronous reset active high


	-- Choose between signed or unsigned representation
	SIGNED_REPRESENTATION_U_SIGNED_N: in std_logic;
		-- '0'  	Unsigned
		-- '1'  	Signed


	-- Input registers
	REG237: in std_logic_vector(7 downto 0);
	REG238: in std_logic_vector(7 downto 0);
	REG239: in std_logic_vector(7 downto 0);
	REG240: in std_logic_vector(7 downto 0);
	REG241: in std_logic_vector(7 downto 0);
	REG242: in std_logic_vector(7 downto 0);
	REG243: in std_logic_vector(7 downto 0);
	REG244: in std_logic_vector(7 downto 0);
	REG245: in std_logic_vector(7 downto 0);
	REG246: in std_logic_vector(7 downto 0);
	REG247: in std_logic_vector(7 downto 0);
	REG248: in std_logic_vector(7 downto 0);
	REG249: in std_logic_vector(7 downto 0);


	-- Trace 1 input signals
	-- 1-bit signals
	SIGNAL_1_BIT_1_1: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_1_1: in std_logic;
	SIGNAL_1_BIT_1_2: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_1_2: in std_logic;
	SIGNAL_1_BIT_1_3: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_1_3: in std_logic;
	SIGNAL_1_BIT_1_4: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_1_4: in std_logic;

	-- 2-bit signals
	SIGNAL_2_BIT_1_1: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_1_1: in std_logic;
	SIGNAL_2_BIT_1_2: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_1_2: in std_logic;
	SIGNAL_2_BIT_1_3: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_1_3: in std_logic;
	SIGNAL_2_BIT_1_4: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_1_4: in std_logic;

	-- 4-bit signals
	SIGNAL_4_BIT_1_1: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_1_1: in std_logic;
	SIGNAL_4_BIT_1_2: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_1_2: in std_logic;
	SIGNAL_4_BIT_1_3: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_1_3: in std_logic;
	SIGNAL_4_BIT_1_4: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_1_4: in std_logic;

	-- 8-bit signals
	SIGNAL_8_BIT_1_1: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_1_1: in std_logic;
	SIGNAL_8_BIT_1_2: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_1_2: in std_logic;
	SIGNAL_8_BIT_1_3: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_1_3: in std_logic;
	SIGNAL_8_BIT_1_4: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_1_4: in std_logic;

	-- 16-bit signals
	SIGNAL_16_BIT_1_1: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_1_1: in std_logic;
	SIGNAL_16_BIT_1_2: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_1_2: in std_logic;
	SIGNAL_16_BIT_1_3: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_1_3: in std_logic;
	SIGNAL_16_BIT_1_4: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_1_4: in std_logic;
	-- Add signals as necessary (up to 127 signals total per trace)


	-- Trace 2 input signals
	-- 1-bit signals
	SIGNAL_1_BIT_2_1: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_2_1: in std_logic;
	SIGNAL_1_BIT_2_2: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_2_2: in std_logic;
	SIGNAL_1_BIT_2_3: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_2_3: in std_logic;
	SIGNAL_1_BIT_2_4: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_2_4: in std_logic;

	-- 2-bit signals
	SIGNAL_2_BIT_2_1: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_2_1: in std_logic;
	SIGNAL_2_BIT_2_2: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_2_2: in std_logic;
	SIGNAL_2_BIT_2_3: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_2_3: in std_logic;
	SIGNAL_2_BIT_2_4: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_2_4: in std_logic;

	-- 4-bit signals
	SIGNAL_4_BIT_2_1: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_2_1: in std_logic;
	SIGNAL_4_BIT_2_2: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_2_2: in std_logic;
	SIGNAL_4_BIT_2_3: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_2_3: in std_logic;
	SIGNAL_4_BIT_2_4: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_2_4: in std_logic;

	-- 8-bit signals
	SIGNAL_8_BIT_2_1: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_2_1: in std_logic;
	SIGNAL_8_BIT_2_2: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_2_2: in std_logic;
	SIGNAL_8_BIT_2_3: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_2_3: in std_logic;
	SIGNAL_8_BIT_2_4: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_2_4: in std_logic;

	-- 16-bit signals
	SIGNAL_16_BIT_2_1: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_2_1: in std_logic;
	SIGNAL_16_BIT_2_2: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_2_2: in std_logic;
	SIGNAL_16_BIT_2_3: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_2_3: in std_logic;
	SIGNAL_16_BIT_2_4: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_2_4: in std_logic;
	-- Add signals as necessary (up to 127 signals total per trace)


	-- Trace 3 input signals
	-- 1-bit signals
	SIGNAL_1_BIT_3_1: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_3_1: in std_logic;
	SIGNAL_1_BIT_3_2: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_3_2: in std_logic;
	SIGNAL_1_BIT_3_3: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_3_3: in std_logic;
	SIGNAL_1_BIT_3_4: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_3_4: in std_logic;

	-- 2-bit signals
	SIGNAL_2_BIT_3_1: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_3_1: in std_logic;
	SIGNAL_2_BIT_3_2: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_3_2: in std_logic;
	SIGNAL_2_BIT_3_3: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_3_3: in std_logic;
	SIGNAL_2_BIT_3_4: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_3_4: in std_logic;

	-- 4-bit signals
	SIGNAL_4_BIT_3_1: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_3_1: in std_logic;
	SIGNAL_4_BIT_3_2: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_3_2: in std_logic;
	SIGNAL_4_BIT_3_3: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_3_3: in std_logic;
	SIGNAL_4_BIT_3_4: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_3_4: in std_logic;

	-- 8-bit signals
	SIGNAL_8_BIT_3_1: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_3_1: in std_logic;
	SIGNAL_8_BIT_3_2: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_3_2: in std_logic;
	SIGNAL_8_BIT_3_3: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_3_3: in std_logic;
	SIGNAL_8_BIT_3_4: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_3_4: in std_logic;

	-- 16-bit signals
	SIGNAL_16_BIT_3_1: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_3_1: in std_logic;
	SIGNAL_16_BIT_3_2: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_3_2: in std_logic;
	SIGNAL_16_BIT_3_3: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_3_3: in std_logic;
	SIGNAL_16_BIT_3_4: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_3_4: in std_logic;
	-- Add signals as necessary (up to 127 signals total per trace)


	-- Trace 4 input signals
	-- 1-bit signals
	SIGNAL_1_BIT_4_1: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_4_1: in std_logic;
	SIGNAL_1_BIT_4_2: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_4_2: in std_logic;
	SIGNAL_1_BIT_4_3: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_4_3: in std_logic;
	SIGNAL_1_BIT_4_4: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_4_4: in std_logic;

	-- 2-bit signals
	SIGNAL_2_BIT_4_1: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_4_1: in std_logic;
	SIGNAL_2_BIT_4_2: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_4_2: in std_logic;
	SIGNAL_2_BIT_4_3: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_4_3: in std_logic;
	SIGNAL_2_BIT_4_4: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_4_4: in std_logic;

	-- 4-bit signals
	SIGNAL_4_BIT_4_1: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_4_1: in std_logic;
	SIGNAL_4_BIT_4_2: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_4_2: in std_logic;
	SIGNAL_4_BIT_4_3: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_4_3: in std_logic;
	SIGNAL_4_BIT_4_4: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_4_4: in std_logic;

	-- 8-bit signals
	SIGNAL_8_BIT_4_1: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_4_1: in std_logic;
	SIGNAL_8_BIT_4_2: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_4_2: in std_logic;
	SIGNAL_8_BIT_4_3: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_4_3: in std_logic;
	SIGNAL_8_BIT_4_4: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_4_4: in std_logic;

	-- 16-bit signals
	SIGNAL_16_BIT_4_1: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_4_1: in std_logic;
	SIGNAL_16_BIT_4_2: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_4_2: in std_logic;
	SIGNAL_16_BIT_4_3: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_4_3: in std_logic;
	SIGNAL_16_BIT_4_4: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_4_4: in std_logic;
	-- Add signals as necessary (up to 127 signals total per trace)



	-- Trigger signals
	-- 1-bit Triggers
	TRIGGER_1_BIT_1: in std_logic;
	TRIGGER_1_BIT_SAMPLE_CLK_1: in std_logic;
	TRIGGER_1_BIT_2: in std_logic;
	TRIGGER_1_BIT_SAMPLE_CLK_2: in std_logic;
	TRIGGER_1_BIT_3: in std_logic;
	TRIGGER_1_BIT_SAMPLE_CLK_3: in std_logic;
	TRIGGER_1_BIT_4: in std_logic;
	TRIGGER_1_BIT_SAMPLE_CLK_4: in std_logic;

	-- 2-bit Triggers
	TRIGGER_2_BIT_1: in std_logic_vector(1 downto 0);
	TRIGGER_2_BIT_SAMPLE_CLK_1: in std_logic;
	TRIGGER_2_BIT_2: in std_logic_vector(1 downto 0);
	TRIGGER_2_BIT_SAMPLE_CLK_2: in std_logic;
	TRIGGER_2_BIT_3: in std_logic_vector(1 downto 0);
	TRIGGER_2_BIT_SAMPLE_CLK_3: in std_logic;
	TRIGGER_2_BIT_4: in std_logic_vector(1 downto 0);
	TRIGGER_2_BIT_SAMPLE_CLK_4: in std_logic;

	-- 4-bit Triggers
	TRIGGER_4_BIT_1: in std_logic_vector(3 downto 0);
	TRIGGER_4_BIT_SAMPLE_CLK_1: in std_logic;
	TRIGGER_4_BIT_2: in std_logic_vector(3 downto 0);
	TRIGGER_4_BIT_SAMPLE_CLK_2: in std_logic;
	TRIGGER_4_BIT_3: in std_logic_vector(3 downto 0);
	TRIGGER_4_BIT_SAMPLE_CLK_3: in std_logic;
	TRIGGER_4_BIT_4: in std_logic_vector(3 downto 0);
	TRIGGER_4_BIT_SAMPLE_CLK_4: in std_logic;

	-- 8-bit Triggers
	TRIGGER_8_BIT_1: in std_logic_vector(7 downto 0);
	TRIGGER_8_BIT_SAMPLE_CLK_1: in std_logic;
	TRIGGER_8_BIT_2: in std_logic_vector(7 downto 0);
	TRIGGER_8_BIT_SAMPLE_CLK_2: in std_logic;
	TRIGGER_8_BIT_3: in std_logic_vector(7 downto 0);
	TRIGGER_8_BIT_SAMPLE_CLK_3: in std_logic;
	TRIGGER_8_BIT_4: in std_logic_vector(7 downto 0);
	TRIGGER_8_BIT_SAMPLE_CLK_4: in std_logic;

	-- 16-bit Triggers
	TRIGGER_16_BIT_1: in std_logic_vector(15 downto 0);
	TRIGGER_16_BIT_SAMPLE_CLK_1: in std_logic;
	TRIGGER_16_BIT_2: in std_logic_vector(15 downto 0);
	TRIGGER_16_BIT_SAMPLE_CLK_2: in std_logic;
	TRIGGER_16_BIT_3: in std_logic_vector(15 downto 0);
	TRIGGER_16_BIT_SAMPLE_CLK_3: in std_logic;
	TRIGGER_16_BIT_4: in std_logic_vector(15 downto 0);
	TRIGGER_16_BIT_SAMPLE_CLK_4: in std_logic;
	-- Add triggers as necessary (up to 127 trigger signals total)


	-- Output registers
	REG250: out std_logic_vector(7 downto 0);
	REG251: out std_logic_vector(7 downto 0);

	-- Other input signals
	TRIGGER_REARM_TOGGLE: in std_logic;
	FORCE_TRIGGER_TOGGLE: in std_logic;
	REG250_READ: in std_logic;
	START_CAPTURE_TOGGLE: in std_logic
);
end component;



--------------------------------------------------------
--     ATTRIBUTES
--------------------------------------------------------
-- DCM attributes
attribute DLL_FREQUENCY_MODE : string; 
attribute DUTY_CYCLE_CORRECTION : string;
attribute CLKOUT_PHASE_SHIFT : string;
attribute PHASE_SHIFT  : integer;
attribute CLKFX_MULTIPLY : integer;
attribute CLKFX_DIVIDE : integer; 
attribute STARTUP_WAIT : string; 

attribute DLL_FREQUENCY_MODE of U_DCM: label is "LOW";
	-- low frequency mode (until about 210 MHz) see Virtex-II switching characteristics specifications
attribute DUTY_CYCLE_CORRECTION of U_DCM: label is "TRUE";
	-- make duty cycle 50/50
attribute CLKOUT_PHASE_SHIFT of U_DCM: label is "NONE";
	-- phase shifter is not being used
attribute PHASE_SHIFT  of U_DCM: label is 0;
attribute CLKFX_MULTIPLY of U_DCM: label is 6;
 	-- generate clock at frequency fin * M / D
attribute CLKFX_DIVIDE of U_DCM: label is 2;
attribute STARTUP_WAIT of U_DCM: label is "FALSE";
 	-- do not wait for DCM to lock to start FPGA

--------------------------------------------------------
--     SIGNALS
--------------------------------------------------------
-- Suffix _D indicates a one CLK delayed version of the net with the same name
-- Suffix _E indicates an extended precision version of the net with the same name
-- Suffix _N indicates an inverted version of the net with the same name
--// CONSTANTS
signal ZERO: std_logic;
signal ONE: std_logic;
signal ZERO1: std_logic_vector(0 downto 0);
signal ZERO2: std_logic_vector(1 downto 0);
signal ZERO4: std_logic_vector(3 downto 0);
signal ZERO8: std_logic_vector(7 downto 0);
signal ZERO16: std_logic_vector(15 downto 0);

--// CLOCKS, RESETS
signal CLK_IN1_A:	std_logic;
signal CLK_IN2_A:	std_logic;
signal CLK0_W: std_logic;	-- deskewed 40 MHz, before going global
signal CLKFX_W: std_logic;	-- synthesized processing clock before going global
signal CLK_P: std_logic;
signal CLK_IO: std_logic;
signal USB_CLK60G: std_logic;
signal DCM_LOCKED: std_logic;
signal COUNTER8:  std_logic_vector(2 downto 0);
signal ASYNC_RESET: std_logic;
	-- asynchronous reset
signal RESET_COUNTER: std_logic_vector(5 downto 0) := "000000";	-- initialize for simulations

--// RECLOCKED INPUTS
signal J3_D: std_logic_vector(34 downto 2);

--// USB signals
signal USB_TP: std_logic_vector(10 downto 1);

--// ComScope traces
signal S1_1: std_logic_vector(7 downto 0);	
signal S1_1_CLK: std_logic;
signal S1_1B_CLK: std_logic;
signal S1_2: std_logic_vector(7 downto 0);	
signal S1_2_CLK: std_logic;
signal S1_2B_CLK: std_logic;
signal S1_3: std_logic_vector(7 downto 0);	
signal S1_3_CLK: std_logic;
signal S1_3B_CLK: std_logic;
signal S1_4: std_logic_vector(7 downto 0);	
signal S1_4_CLK: std_logic;
signal S1_4B_CLK: std_logic;

signal S2_1: std_logic;
signal S2_1_CLK: std_logic;
signal S2_1B_CLK: std_logic;
signal S2_2: std_logic;
signal S2_2_CLK: std_logic;
signal S2_2B_CLK: std_logic;
signal S2_3: std_logic;
signal S2_3_CLK: std_logic;
signal S2_3B_CLK: std_logic;
signal S2_4: std_logic;
signal S2_4_CLK: std_logic;
signal S2_4B_CLK: std_logic;

signal S3_1: std_logic_vector(7 downto 0);	
signal S3_1_CLK: std_logic;
signal S3_1B_CLK: std_logic;
signal S3_2: std_logic_vector(7 downto 0);	
signal S3_2_CLK: std_logic;
signal S3_2B_CLK: std_logic;
signal S3_3: std_logic_vector(7 downto 0);	
signal S3_3_CLK: std_logic;
signal S3_3B_CLK: std_logic;
signal S3_4: std_logic_vector(7 downto 0);	
signal S3_4_CLK: std_logic;
signal S3_4B_CLK: std_logic;

signal T1: std_logic;
signal T1_CLK: std_logic;
signal T2: std_logic;
signal T2_CLK: std_logic;
signal T3: std_logic;
signal T3_CLK: std_logic;
signal T4: std_logic;
signal T4_CLK: std_logic;

signal TRIGGER_REARM_TOGGLE: std_logic:= '0';
signal FORCE_TRIGGER_TOGGLE: std_logic:= '0';
signal START_CAPTURE_TOGGLE: std_logic:= '0';
signal REG250_READ: std_logic;

--// Atmel uc interface
signal UC_ALE: std_logic;
signal UC_RDN_D: std_logic;
signal UC_RDN_D2: std_logic;
signal UC_RDN_D3: std_logic;
signal UC_WRN: std_logic;
signal UC_WRN_D: std_logic;
signal UC_WRN_D2: std_logic;
signal UC_WRN_D3: std_logic;
signal UC_ADDRESS: std_logic_vector(7 downto 0);
signal UC_ADDR_D: std_logic_vector(7 downto 0);
signal UC_ADDR_D2: std_logic_vector(7 downto 0);
signal UC_ADDR_D3: std_logic_vector(7 downto 0);
signal CONFIG_CHANGE_TOGGLE: std_logic;
signal CONFIG_CHANGE_TOGGLE_D: std_logic;
signal CONFIG_CHANGE_TOGGLE_D2: std_logic;
signal CONFIG_CHANGE_PULSE: std_logic;

--// Control Registers
signal REG0: std_logic_vector(7 downto 0);
signal REG1: std_logic_vector(7 downto 0);
signal REG2: std_logic_vector(7 downto 0);
signal REG3: std_logic_vector(7 downto 0);
signal REG4: std_logic_vector(7 downto 0);
signal REG5: std_logic_vector(7 downto 0);
signal REG6: std_logic_vector(7 downto 0);
signal REG7: std_logic_vector(7 downto 0);
signal REG8: std_logic_vector(7 downto 0);
signal REG9: std_logic_vector(7 downto 0);
signal REG10: std_logic_vector(7 downto 0);
signal REG11: std_logic_vector(7 downto 0);
signal REG12: std_logic_vector(7 downto 0);
signal REG13: std_logic_vector(7 downto 0);
signal REG14: std_logic_vector(7 downto 0);
signal REG15: std_logic_vector(7 downto 0);
signal REG16: std_logic_vector(7 downto 0);
signal REG17: std_logic_vector(7 downto 0);
signal REG18: std_logic_vector(7 downto 0);
signal REG19: std_logic_vector(7 downto 0);
signal REG20: std_logic_vector(7 downto 0);
signal REG21: std_logic_vector(7 downto 0);
signal REG22: std_logic_vector(7 downto 0);
signal REG23: std_logic_vector(7 downto 0);
signal REG24: std_logic_vector(7 downto 0);
signal REG25: std_logic_vector(7 downto 0);
signal REG26: std_logic_vector(7 downto 0);
signal REG27: std_logic_vector(7 downto 0);
signal REG28: std_logic_vector(7 downto 0);
signal REG29: std_logic_vector(7 downto 0);
signal REG30: std_logic_vector(7 downto 0);
signal REG31: std_logic_vector(7 downto 0);
signal REG32: std_logic_vector(7 downto 0);
signal REG33: std_logic_vector(7 downto 0);
signal REG34: std_logic_vector(7 downto 0);
signal REG35: std_logic_vector(7 downto 0);
signal REG36: std_logic_vector(7 downto 0);
signal REG37: std_logic_vector(7 downto 0);
signal REG38: std_logic_vector(7 downto 0);
signal REG254: std_logic_vector(7 downto 0);
signal REG255: std_logic_vector(7 downto 0);

signal TX_USB_TOGGLE: std_logic;
signal TX_USB_TOGGLE_D: std_logic;
signal TX_USB_TOGGLE_D2: std_logic;
signal RX_USB_TOGGLE: std_logic;
signal RX_USB_TOGGLE_D: std_logic;
signal RX_USB_TOGGLE_D2: std_logic;
signal DATA1_OUT_SAMPLE_CLK: std_logic;
signal DATA1_OUT_SAMPLE_CLK_REQ: std_logic;
signal DATA1_OUT_BUFFER_EMPTY: std_logic;
signal DATA1_OUT: std_logic_vector(7 downto 0);
signal DATA1_IN: std_logic_vector(7 downto 0);
signal DATA1_IN_SAMPLE_CLK: std_logic;
signal DATA1_IN_SAMPLE_CLK_REQ: std_logic;
signal DATA2_OUT_SAMPLE_CLK: std_logic;
signal DATA2_OUT_SAMPLE_CLK_REQ: std_logic;
signal DATA2_OUT_BUFFER_EMPTY: std_logic;
signal DATA2_OUT: std_logic_vector(7 downto 0);
signal DATA2_IN: std_logic_vector(7 downto 0);
signal DATA2_IN_SAMPLE_CLK: std_logic;
signal DATA2_IN_SAMPLE_CLK_REQ: std_logic;


--// ComScope control registers
signal REG237: std_logic_vector(7 downto 0):= "00000000";
signal REG238: std_logic_vector(7 downto 0):= "00000000";
signal REG239: std_logic_vector(7 downto 0):= "00000000";
signal REG240: std_logic_vector(7 downto 0):= "00001101";	-- Trace 1 signal selection
signal REG241: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG242: std_logic_vector(7 downto 0):= "00001110";	-- Trace 2 signal selection
signal REG243: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG244: std_logic_vector(7 downto 0):= "00000000";	-- Trace 3 signal selection
signal REG245: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG246: std_logic_vector(7 downto 0):= "00000000";	-- Trace 4 signal selection
signal REG247: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG248: std_logic_vector(7 downto 0):= "10001101";	-- Trigger edge, trigger signal
signal REG249: std_logic_vector(7 downto 0):= "10111111";	-- Trigger threshold
signal REG250: std_logic_vector(7 downto 0):= "00000000";
signal REG251: std_logic_vector(7 downto 0):= "00000000";


--// FPGA option and revision (ASCII format)
-- These two ASCII characters appear in the Comblock Control Center.
-- '0' = 48
-- 'A' = 65
constant OPTION: std_logic_vector(7 downto 0) := x"20";	-- '-'
constant REVISION: std_logic_vector(7 downto 0) := x"39";	-- '9'
--------------------------------------------------------
--      IMPLEMENTATION
--------------------------------------------------------

begin
--// CONSTANTS ---------------------
ZERO <= '0';
ONE <= '1';
ZERO1 <= "0";
ZERO2 <= "00";
ZERO4 <= (others => '0');
ZERO8 <= (others => '0');
ZERO16 <= (others => '0');

--// RESET ---------------------
-- Generate a reset pulse after power up. 
-- The reset counter starts at 0 and counts to 32 where it stays.
-- A one CLK wide reset pulse is generated when the counter equals 31.
RESET_COUNTER_GEN_001: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		if(RESET_COUNTER(5) = '0') then
			RESET_COUNTER <= RESET_COUNTER + 1;
	  	end if;
  	end if;
end process;

RESET_COUNTER_GEN_002: process(CLK_P)
begin
	if rising_edge(CLK_P) then
		if(RESET_COUNTER = "011111") then
			ASYNC_RESET <= '1';
	  	else
			ASYNC_RESET <= '0';
	  	end if;
  	end if;
end process;

--// CLOCKS ---------------------
-- Use input buffers for clocks
IBUF_001: IBUFG port map(
	I => CLK_IN1,
	O => CLK_IN1_A
 	);

IBUF_002: IBUFG port map(
	I => CLK_IN2,
	O => CLK_IN2_A
 	);

-- Global buffers
CLK_GEN_001: BUFG  port map (I=> CLKFX_W, O=> CLK_P);	
	-- synthesized clock for internal processing
CLK_GEN_002: BUFG  port map (I=> CLK0_W, O=> CLK_IO);	
	-- 40 MHz clock for I/Os
BUFG_002: BUFG port map(I => UC_WRN_IN, O => UC_WRN);  
BUFG_003: BUFG port map(I => UC_ALE_IN, O => UC_ALE);  
-- Master clock originates from the SMSC GT3200 USB2.0 PHY IC. 60 MHz.
BUFG_004: BUFG port map(I => USB_CLKOUT, O => USB_CLK60G);


-- DCM used as frequency synthesizer. 40 MHz x M / D = 120 MHz, where M = 3, D = 2.
-- M and D are set through the attributes above.
U_DCM: DCM
  port map (
            CLKIN =>    CLK_IN1,  
					-- Select reference clock (internal CLK_IN1 or external CLK_IN2)
            CLKFB =>    CLK_IO,
            DSSEN =>    ZERO,
            PSINCDEC => ZERO,
            PSEN =>     ZERO,
            PSCLK =>    ZERO,
            RST =>      ZERO,
				CLK0  =>    CLK0_W,	-- deskewed version of the 40 MHz reference clock
 				CLKFX =>    CLKFX_W,	-- synthesized output. 50/50 duty cycle.
            LOCKED =>   DCM_LOCKED
	   ); 

-- Test clocks. Generate a divide-by-8 clock and forward to 
-- the INIT# test point (located next to the 'DONE' test point)
-- This is an easy way to verify that the VHDL code is properly implemented
-- and that no glitches exist on an external or internal clock.
process(ASYNC_RESET, CLK_P)
begin
	if(ASYNC_RESET = '1') then
		COUNTER8 <= "000";
	elsif rising_edge(CLK_P) then
		COUNTER8 <= COUNTER8 + 1;
	end if;
end process;
INITB <= COUNTER8(2);

--// Reclock input signals
-- It is always a good idea to reclock external input signals to minimize 
-- propagation delay between modules.
RECLOCK_001: process(ASYNC_RESET, CLK_IN2_A)
begin
	if(ASYNC_RESET = '1') then
		J3_D <= (others => '0');
	elsif rising_edge(CLK_IN2_A) then
		J3_D <= J3;
	end if;
end process;

--// Forward reference clock to other ComBlock modules.
J1(1) <= CLK_IO;


--// DEFINE YOUR TEST POINTS HERE -----------------------
TEST_POINTS(4 downto 1) <= USB_TP(4 downto 1);
TEST_POINTS(5) <= USB_VBUS_SENSE;			
TEST_POINTS(6) <= USB_RXERROR;			
TEST_POINTS(7) <= USB_CLK60G;		
TEST_POINTS(8) <= USB_RXVALID;
TEST_POINTS(9) <= USB_RXACTIVE;
TEST_POINTS(10) <= ASYNC_RESET;

--// USB interface
USB_INTERFACE_001: USB20 port map(
	ASYNC_RESET => ASYNC_RESET,
	USB_CLK60G => USB_CLK60G,
	CLK_P => CLK_P,

	-- interface with USB PHY. Direct connection to I/O pins.
	USB_RESET => USB_RESET,
	USB_DATABUS16_8 => USB_DATABUS16_8,
	USB_SUSPENDN => USB_SUSPENDN,
	USB_XCVRSELECT => USB_XCVRSELECT,
	USB_TERMSELECT => USB_TERMSELECT,
	USB_OPMODE => USB_OPMODE,
	USB_LINESTATE => USB_LINESTATE,
	USB_TXVALID => USB_TXVALID,
	USB_TXREADY => USB_TXREADY,
	USB_RXVALID => USB_RXVALID,
	USB_RXACTIVE => USB_RXACTIVE,
	USB_RXERROR => USB_RXERROR,
	USB_DATA_IN => USB_DATA_IN,
	USB_DATA_OUT => USB_DATA_OUT,
	USB_VBUS_SENSE => USB_VBUS_SENSE,

	DATA1_OUT => DATA1_OUT,
	DATA1_OUT_BUFFER_EMPTY => DATA1_OUT_BUFFER_EMPTY,
	DATA1_OUT_SAMPLE_CLK => DATA1_OUT_SAMPLE_CLK,
	DATA1_OUT_SAMPLE_CLK_REQ => DATA1_OUT_SAMPLE_CLK_REQ,
	DATA1_IN =>	DATA1_IN,
	DATA1_IN_SAMPLE_CLK => DATA1_IN_SAMPLE_CLK,
	DATA1_IN_SAMPLE_CLK_REQ =>	DATA1_IN_SAMPLE_CLK_REQ,

	DATA2_OUT => DATA2_OUT,
	DATA2_OUT_BUFFER_EMPTY => DATA2_OUT_BUFFER_EMPTY,
	DATA2_OUT_SAMPLE_CLK => DATA2_OUT_SAMPLE_CLK,
	DATA2_OUT_SAMPLE_CLK_REQ => DATA2_OUT_SAMPLE_CLK_REQ,
	DATA2_IN =>	DATA2_IN,
	DATA2_IN_SAMPLE_CLK => DATA2_IN_SAMPLE_CLK,
	DATA2_IN_SAMPLE_CLK_REQ =>	DATA2_IN_SAMPLE_CLK_REQ,

	USB_TP => USB_TP
);

--// INSERT YOUR VHDL CODE HERE -----------

--// Assign status registers here (the following is a sample)
--			REG15 <= DATA1_OUT;
--			REG16 <= DATA2_OUT;

--// Comscope --------------
-- Define Comscope traces here 
-- Trace 1. 8-bit wide. Up to 4 signals
--S1_1_CLK <= USB_RXACTIVE and USB_RXVALID;
S1_1_CLK <= '1';
-- Trace 2. 1-bit wide. Up to 4 signals
S2_1_CLK <= '1';
S2_2_CLK <= '1';
S2_3_CLK <= '1';
S2_4_CLK <= '1';
-- Trace 3. 8-bit wide. Up to 4 signals
--S3_1 <= "00000000";
--S3_1_CLK <= '1';
-- Trigger. 1-bit wide. Up to 4 signals.
T1_CLK <= '1';


S1_1B_CLK <= S1_1_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock
S1_2B_CLK <= S1_2_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock
S1_3B_CLK <= S1_3_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock
S1_4B_CLK <= S1_4_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock

S2_1B_CLK <= S2_1_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock
S2_2B_CLK <= S2_2_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock
S2_3B_CLK <= S2_3_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock
S2_4B_CLK <= S2_4_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock

S3_1B_CLK <= S3_1_CLK or REG244(7);	-- Nominal sampling clock or real-time processing clock
S3_2B_CLK <= S3_2_CLK or REG244(7);	-- Nominal sampling clock or real-time processing clock
S3_3B_CLK <= S3_3_CLK or REG244(7);	-- Nominal sampling clock or real-time processing clock
S3_4B_CLK <= S1_4_CLK or REG244(7);	-- Nominal sampling clock or real-time processing clock

COMSCOPE_001: COMSCOPE port map(
   CLK => USB_CLK60G,
	ASYNC_RESET => ASYNC_RESET,
	SIGNED_REPRESENTATION_U_SIGNED_N => REG248(6),	-- trigger format signed(1) or unsigned(0)
	REG237 => REG237,
	REG238 => REG238,
	REG239 => REG239,
	REG240 => REG240,
	REG241 => REG241,
	REG242 => REG242,
	REG243 => REG243,
	REG244 => REG244,
	REG245 => REG245,
	REG246 => REG246,
	REG247 => REG247,
	REG248 => REG248,
	REG249 => REG249,
	SIGNAL_1_BIT_1_1 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_1_1 => ZERO,
	SIGNAL_1_BIT_1_2 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_1_2 => ZERO,
	SIGNAL_1_BIT_1_3 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_1_3 => ZERO,
	SIGNAL_1_BIT_1_4 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_1_4 => ZERO,
	SIGNAL_2_BIT_1_1 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_1_1 => ZERO,
	SIGNAL_2_BIT_1_2 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_1_2 => ZERO,
	SIGNAL_2_BIT_1_3 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_1_3 => ZERO,
	SIGNAL_2_BIT_1_4 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_1_4 => ZERO,
	SIGNAL_4_BIT_1_1 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_1_1 => ZERO,
	SIGNAL_4_BIT_1_2 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_1_2 => ZERO,
	SIGNAL_4_BIT_1_3 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_1_3 => ZERO,
	SIGNAL_4_BIT_1_4 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_1_4 => ZERO,
	SIGNAL_8_BIT_1_1 => S1_1,
	SIGNAL_8_BIT_SAMPLE_CLK_1_1 => S1_1B_CLK,
	SIGNAL_8_BIT_1_2 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_1_2 => ZERO,
	SIGNAL_8_BIT_1_3 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_1_3 => ZERO,
	SIGNAL_8_BIT_1_4 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_1_4 => ZERO,
	SIGNAL_16_BIT_1_1 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_1_1 => ZERO,
	SIGNAL_16_BIT_1_2 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_1_2 => ZERO,
	SIGNAL_16_BIT_1_3 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_1_3 => ZERO,
	SIGNAL_16_BIT_1_4 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_1_4 => ZERO,
	SIGNAL_1_BIT_2_1 => S2_1,
	SIGNAL_1_BIT_SAMPLE_CLK_2_1 => S2_1B_CLK,
	SIGNAL_1_BIT_2_2 => S2_2,
	SIGNAL_1_BIT_SAMPLE_CLK_2_2 => S2_2B_CLK,
	SIGNAL_1_BIT_2_3 => S2_3,
	SIGNAL_1_BIT_SAMPLE_CLK_2_3 => S2_3B_CLK,
	SIGNAL_1_BIT_2_4 => S2_4,
	SIGNAL_1_BIT_SAMPLE_CLK_2_4 => S2_4B_CLK,
	SIGNAL_2_BIT_2_1 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_2_1 => ZERO,
	SIGNAL_2_BIT_2_2 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_2_2 => ZERO,
	SIGNAL_2_BIT_2_3 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_2_3 => ZERO,
	SIGNAL_2_BIT_2_4 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_2_4 => ZERO,
	SIGNAL_4_BIT_2_1 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_2_1 => ZERO,
	SIGNAL_4_BIT_2_2 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_2_2 => ZERO,
	SIGNAL_4_BIT_2_3 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_2_3 => ZERO,
	SIGNAL_4_BIT_2_4 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_2_4 => ZERO,
	SIGNAL_8_BIT_2_1 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_2_1 => ZERO,
	SIGNAL_8_BIT_2_2 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_2_2 => ZERO,
	SIGNAL_8_BIT_2_3 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_2_3 => ZERO,
	SIGNAL_8_BIT_2_4 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_2_4 => ZERO,
	SIGNAL_16_BIT_2_1 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_2_1 => ZERO,
	SIGNAL_16_BIT_2_2 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_2_2 => ZERO,
	SIGNAL_16_BIT_2_3 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_2_3 => ZERO,
	SIGNAL_16_BIT_2_4 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_2_4 => ZERO,
	SIGNAL_1_BIT_3_1 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_3_1 => ZERO,
	SIGNAL_1_BIT_3_2 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_3_2 => ZERO,
	SIGNAL_1_BIT_3_3 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_3_3 => ZERO,
	SIGNAL_1_BIT_3_4 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_3_4 => ZERO,
	SIGNAL_2_BIT_3_1 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_3_1 => ZERO,
	SIGNAL_2_BIT_3_2 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_3_2 => ZERO,
	SIGNAL_2_BIT_3_3 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_3_3 => ZERO,
	SIGNAL_2_BIT_3_4 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_3_4 => ZERO,
	SIGNAL_4_BIT_3_1 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_3_1 => ZERO,
	SIGNAL_4_BIT_3_2 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_3_2 => ZERO,
	SIGNAL_4_BIT_3_3 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_3_3 => ZERO,
	SIGNAL_4_BIT_3_4 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_3_4 => ZERO,
	SIGNAL_8_BIT_3_1 => S3_1,
	SIGNAL_8_BIT_SAMPLE_CLK_3_1 => S3_1_CLK,
	SIGNAL_8_BIT_3_2 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_3_2 => ZERO,
	SIGNAL_8_BIT_3_3 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_3_3 => ZERO,
	SIGNAL_8_BIT_3_4 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_3_4 => ZERO,
	SIGNAL_16_BIT_3_1 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_3_1 => ZERO,
	SIGNAL_16_BIT_3_2 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_3_2 => ZERO,
	SIGNAL_16_BIT_3_3 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_3_3 => ZERO,
	SIGNAL_16_BIT_3_4 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_3_4 => ZERO,
	SIGNAL_1_BIT_4_1 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_4_1 => ZERO,
	SIGNAL_1_BIT_4_2 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_4_2 => ZERO,
	SIGNAL_1_BIT_4_3 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_4_3 => ZERO,
	SIGNAL_1_BIT_4_4 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_4_4 => ZERO,
	SIGNAL_2_BIT_4_1 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_4_1 => ZERO,
	SIGNAL_2_BIT_4_2 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_4_2 => ZERO,
	SIGNAL_2_BIT_4_3 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_4_3 => ZERO,
	SIGNAL_2_BIT_4_4 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_4_4 => ZERO,
	SIGNAL_4_BIT_4_1 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_4_1 => ZERO,
	SIGNAL_4_BIT_4_2 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_4_2 => ZERO,
	SIGNAL_4_BIT_4_3 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_4_3 => ZERO,
	SIGNAL_4_BIT_4_4 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_4_4 => ZERO,
	SIGNAL_8_BIT_4_1 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_4_1 => ZERO,
	SIGNAL_8_BIT_4_2 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_4_2 => ZERO,
	SIGNAL_8_BIT_4_3 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_4_3 => ZERO,
	SIGNAL_8_BIT_4_4 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_4_4 => ZERO,
	SIGNAL_16_BIT_4_1 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_4_1 => ZERO,
	SIGNAL_16_BIT_4_2 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_4_2 => ZERO,
	SIGNAL_16_BIT_4_3 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_4_3 => ZERO,
	SIGNAL_16_BIT_4_4 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_4_4 => ZERO,
	TRIGGER_1_BIT_1 => T1,
	TRIGGER_1_BIT_SAMPLE_CLK_1 => T1_CLK,
	TRIGGER_1_BIT_2 => ZERO,
	TRIGGER_1_BIT_SAMPLE_CLK_2 => ZERO,
	TRIGGER_1_BIT_3 => ZERO,
	TRIGGER_1_BIT_SAMPLE_CLK_3 => ZERO,
	TRIGGER_1_BIT_4 => ZERO,
	TRIGGER_1_BIT_SAMPLE_CLK_4 => ZERO,
	TRIGGER_2_BIT_1 => ZERO2,
	TRIGGER_2_BIT_SAMPLE_CLK_1 => ZERO,
	TRIGGER_2_BIT_2 => ZERO2,
	TRIGGER_2_BIT_SAMPLE_CLK_2 => ZERO,
	TRIGGER_2_BIT_3 => ZERO2,
	TRIGGER_2_BIT_SAMPLE_CLK_3 => ZERO,
	TRIGGER_2_BIT_4 => ZERO2,
	TRIGGER_2_BIT_SAMPLE_CLK_4 => ZERO,
	TRIGGER_4_BIT_1 => ZERO4,
	TRIGGER_4_BIT_SAMPLE_CLK_1 => ZERO,
	TRIGGER_4_BIT_2 => ZERO4,
	TRIGGER_4_BIT_SAMPLE_CLK_2 => ZERO,
	TRIGGER_4_BIT_3 => ZERO4,
	TRIGGER_4_BIT_SAMPLE_CLK_3 => ZERO,
	TRIGGER_4_BIT_4 => ZERO4,
	TRIGGER_4_BIT_SAMPLE_CLK_4 => ZERO,
	TRIGGER_8_BIT_1 => ZERO8,
	TRIGGER_8_BIT_SAMPLE_CLK_1 => ZERO,
	TRIGGER_8_BIT_2 => ZERO8,
	TRIGGER_8_BIT_SAMPLE_CLK_2 => ZERO,
	TRIGGER_8_BIT_3 => ZERO8,
	TRIGGER_8_BIT_SAMPLE_CLK_3 => ZERO,
	TRIGGER_8_BIT_4 => ZERO8,
	TRIGGER_8_BIT_SAMPLE_CLK_4 => ZERO,
	TRIGGER_16_BIT_1 => ZERO16,
	TRIGGER_16_BIT_SAMPLE_CLK_1 => ZERO,
	TRIGGER_16_BIT_2 => ZERO16,
	TRIGGER_16_BIT_SAMPLE_CLK_2 => ZERO,
	TRIGGER_16_BIT_3 => ZERO16,
	TRIGGER_16_BIT_SAMPLE_CLK_3 => ZERO,
	TRIGGER_16_BIT_4 => ZERO16,
	TRIGGER_16_BIT_SAMPLE_CLK_4 => ZERO,
	REG250 => REG250,
	REG251 => REG251,
	TRIGGER_REARM_TOGGLE => TRIGGER_REARM_TOGGLE,
	FORCE_TRIGGER_TOGGLE => FORCE_TRIGGER_TOGGLE,
	REG250_READ => REG250_READ,
	START_CAPTURE_TOGGLE => START_CAPTURE_TOGGLE
);



--// MONITORING & CONTROL -------------
--// Serial interfaces are used for module-to-module communication.
-- Do not change this section unless indicated in the comments.
-- REG255 reserved for reading 3 serial port inputs
-- These signals are pulled high (to mimic stop bit).
REG255(0) <= RX_L	;	-- port 1 left
REG255(1) <= RX_B	;	-- port 2 bottom
REG255(2) <= RX_R	;	-- port 3 right
REG255(3) <= DATA1_OUT_BUFFER_EMPTY;	
	-- indicates AVAILABILITY of 8-bit byte from USB. Not actual data.
	-- When low, uC must read byte from RX_USB
REG255(7 downto 4) <= "0000";

--// Atmel microcontroller interface
-- Used for monitoring and control.
-- address latch
UC_ADDR_001: process(ASYNC_RESET, UC_ALE, UC_FLASH_CSN, UC_AD)
begin
	if(ASYNC_RESET = '1') then
		UC_ADDRESS <= (others => '0');
	elsif falling_edge(UC_ALE) then
		if(UC_FLASH_CSN = '1') then
			UC_ADDRESS <= UC_AD;
	  	end if;
  	end if;
end process;

RECLOCK_ADDR_D: process(ASYNC_RESET, CLK_P, UC_ADDRESS)
begin
	if(ASYNC_RESET = '1') then
		UC_ADDR_D <= (others =>'0');
		UC_ADDR_D2 <= (others =>'0');
		UC_ADDR_D3 <= (others =>'0');
	elsif rising_edge(CLK_P) then
		UC_ADDR_D <= UC_ADDRESS;
		UC_ADDR_D2 <= UC_ADDR_D;
		UC_ADDR_D3 <= UC_ADDR_D2;
	end if;
end process;


RECLOCK_RDN_D: process(ASYNC_RESET, CLK_P, UC_RDN)
begin
	if(ASYNC_RESET = '1') then
		UC_RDN_D <= '0';
		UC_RDN_D2 <= '0';
		UC_RDN_D3 <= '0';
		DATA1_OUT_SAMPLE_CLK_REQ <= '0';
	elsif rising_edge(CLK_P) then
		UC_RDN_D <= UC_RDN;
		UC_RDN_D2 <= UC_RDN_D;
		UC_RDN_D3 <= UC_RDN_D3;
		if (UC_RDN_D = '1') and (UC_RDN_D2 = '0') and (UC_ADDR_D2 = 254)then
			DATA1_OUT_SAMPLE_CLK_REQ <= '1';
		else
			DATA1_OUT_SAMPLE_CLK_REQ <= '0';
		end if;
	end if;
end process;

-- interface with Atmel uC
-- Reclock TX_USB_TOGGLE
RECLOCK_TX_USB_TOGGLE: process(ASYNC_RESET, CLK_P,TX_USB_TOGGLE)
begin
	if(ASYNC_RESET = '1') then
		TX_USB_TOGGLE_D <= '0';
		TX_USB_TOGGLE_D2 <= '0';
		DATA1_IN_SAMPLE_CLK <= '0';
	elsif rising_edge(CLK_P) then
		TX_USB_TOGGLE_D <= TX_USB_TOGGLE;
		TX_USB_TOGGLE_D2 <= TX_USB_TOGGLE_D;
		if(TX_USB_TOGGLE_D /= TX_USB_TOGGLE_D2) then
			DATA1_IN_SAMPLE_CLK <= '1';
		else
			DATA1_IN_SAMPLE_CLK <= '0';
		end if;
	end if;
end process;

-- read
UC_READ_001: process(UC_FLASH_CSN, UC_RDN, UC_ADDRESS, REG15, REG16, 
	REG17, REG18, REG19, REG20, REG21, REG22, REG23, REG24, REG25, REG26, 
	REG27, REG28, REG29, REG30, REG31, REG32, REG33, REG34, REG35, REG36, 
	REG37, REG38, REG250, REG251, REG255)
begin
	if(UC_FLASH_CSN = '1') and (UC_RDN = '0') then
		if(UC_ADDRESS(7 downto 2) = "111111") then
			if(UC_ADDRESS(1 downto 0) = "11") then
				UC_AD <= REG255;		-- reg 255. Async serial from 3 connectors
			elsif(UC_ADDRESS(1 downto 0) = "10") then
					UC_AD <= DATA1_OUT;		-- reg 254 = 8-bit data from USB
			elsif(UC_ADDRESS(1 downto 0) = "01") then
				UC_AD <= REVISION;	-- reg 253 = fpga revision
			elsif(UC_ADDRESS(1 downto 0) = "00") then
				UC_AD <= OPTION;		-- reg 252 = fpga option
			else
				UC_AD <= "ZZZZZZZZ";
			end if;
		elsif(UC_ADDRESS= 15) then
			UC_AD <= REG15;
		elsif(UC_ADDRESS= 16) then
			UC_AD <= REG16;
		elsif(UC_ADDRESS= 17) then
			UC_AD <= REG17;
		elsif(UC_ADDRESS= 18) then
			UC_AD <= REG18;
		elsif(UC_ADDRESS= 19) then
			UC_AD <= REG19;
		elsif(UC_ADDRESS= 20) then
			UC_AD <= REG20;
		elsif(UC_ADDRESS= 21) then
			UC_AD <= REG21;
		elsif(UC_ADDRESS= 22) then
			UC_AD <= REG22;
		elsif(UC_ADDRESS= 23) then
			UC_AD <= REG23;
		elsif(UC_ADDRESS= 24) then
			UC_AD <= REG24;
		elsif(UC_ADDRESS= 25) then
			UC_AD <= REG25;
		elsif(UC_ADDRESS= 26) then
			UC_AD <= REG26;
		elsif(UC_ADDRESS= 27) then
			UC_AD <= REG27;
		elsif(UC_ADDRESS= 28) then
			UC_AD <= REG28;
		elsif(UC_ADDRESS= 29) then
			UC_AD <= REG29;
		elsif(UC_ADDRESS= 30) then
			UC_AD <= REG30;
		elsif(UC_ADDRESS= 31) then
			UC_AD <= REG31;
		elsif(UC_ADDRESS= 32) then
			UC_AD <= REG32;
		elsif(UC_ADDRESS= 33) then
			UC_AD <= REG33;
		elsif(UC_ADDRESS= 34) then
			UC_AD <= REG34;
		elsif(UC_ADDRESS= 35) then
			UC_AD <= REG35;
		elsif(UC_ADDRESS= 36) then
			UC_AD <= REG36;
		elsif(UC_ADDRESS= 37) then
			UC_AD <= REG37;
		elsif(UC_ADDRESS= 38) then
			UC_AD <= REG38;
		elsif (UC_ADDRESS = 251) then
			UC_AD <= REG251;		-- reg 251 =comscope  status
		elsif (UC_ADDRESS = 250) then
			UC_AD <= REG250;		-- reg 250 = comscope data byte
		else
			UC_AD <= "ZZZZZZZZ";
		end if;
	else
		UC_AD <= "ZZZZZZZZ";
 	end if;
end process;

-- detect when address 250 is being read
REG250_READ <= '1' when (UC_FLASH_CSN = '1') and (UC_RDN = '0') and (UC_ADDRESS = 250) else '0';

-- write. 
-- Registers 0 through 15 are control registers.
UC_WRITE_001: process(ASYNC_RESET, UC_FLASH_CSN, UC_WRN, UC_ADDRESS, UC_AD)
begin
	if(ASYNC_RESET = '1') then
		-- CAREFUL!!!!!! 
		-- when the external clock comes from a bigger FPGA, the ASYNC_RESET
		-- may arrive here AFTER the Atmel uC has configured the registers.
		-- We should avoid resetting the registers.
		TX_L <= '1';	-- stop bit
		TX_B <= '1';
		TX_R <= '1';
		CONFIG_CHANGE_TOGGLE <= '0';
	 elsif falling_edge(UC_WRN) then

		if (UC_ADDRESS(7 downto 4) = "0000") then
			if (UC_ADDRESS(3 downto 0) = "0000") then
				REG0 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = "0001") then
				REG1 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = "0010") then
				REG2 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = "0011") then
				REG3 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = "0100") then
				REG4 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = "0101") then
				REG5 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = "0110") then
				REG6 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = "0111") then
				REG7 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = "1000") then
				REG8 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = "1001") then
				REG9 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = "1010") then
				REG10 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = "1011") then
				REG11 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = "1100") then
				REG12 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = "1101") then
				REG13 <= UC_AD;
			end if;
			if (UC_ADDRESS(3 downto 0) = "1110") then
				REG14 <= UC_AD;
				CONFIG_CHANGE_TOGGLE <= not CONFIG_CHANGE_TOGGLE;	
				-- Configuration change toggles upon writing the last control register
			end if;
		end if;
		if (UC_ADDRESS = 254) then
			-- REG254 write reserved for output serial (asynchronous serial format)
			TX_L <= UC_AD(0);
			TX_B <= UC_AD(1);
			TX_R <= UC_AD(2);
		end if;
		if (UC_ADDRESS = 255) then
			-- REG255 write reserved for output serial (8-bit parallel format)
			DATA1_IN <= UC_AD;
			TX_USB_TOGGLE <= not TX_USB_TOGGLE;
		end if;
	end if;
end process;

--// reserved COMSCOPE
UC_WRITE_002: process(ASYNC_RESET, UC_FLASH_CSN, UC_WRN, UC_ADDRESS, UC_AD, 
	FORCE_TRIGGER_TOGGLE, TRIGGER_REARM_TOGGLE, START_CAPTURE_TOGGLE)
begin
	if(ASYNC_RESET = '1') then
		-- Do not enter default values here. Any default value entered here will be
		-- overwritten by the microprocessor, typically within 10ms after the FPGA is configured.
	elsif falling_edge(UC_WRN) then
		if(UC_FLASH_CSN = '1') then
			-- Reserved for ComScope			
			if (UC_ADDRESS = 237) then
				REG237 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 238) then
				REG238 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 239) then
				REG239 <= UC_AD;
			end if;
			if (UC_ADDRESS = 240) then
				REG240 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 241) then
				REG241 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 242) then
				REG242 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 243) then
				REG243 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 244) then
				REG244 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 245) then
				REG245 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 246) then
				REG246 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 247) then
				REG247 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 248) then
				REG248 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 249) then
				REG249 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 250) then 
				-- detect one-shot force trigger	
				if(UC_AD(0) = '1') then	
					FORCE_TRIGGER_TOGGLE <= not(FORCE_TRIGGER_TOGGLE);
				end if;	
				-- detect one-shot re-arm	
				if(UC_AD(1) = '1') then	
					TRIGGER_REARM_TOGGLE <= not(TRIGGER_REARM_TOGGLE);
				end if;	
				-- start capturing again
				if(UC_AD(2) = '1') then	
					START_CAPTURE_TOGGLE <= not (START_CAPTURE_TOGGLE);
				end if;	
			end if;
		end if;
	end if;
end process;

-- Reclock CONFIG_CHANGE_TOGGLE to be synchronous with processing clock CLK_P
RECLOCK_002: process(ASYNC_RESET, CLK_P)
begin
	if(ASYNC_RESET = '1') then
		CONFIG_CHANGE_TOGGLE_D <= '0';
		CONFIG_CHANGE_TOGGLE_D2 <= '0';
		CONFIG_CHANGE_PULSE <= '0';
	elsif rising_edge(CLK_P) then
		CONFIG_CHANGE_TOGGLE_D <= CONFIG_CHANGE_TOGGLE;
		CONFIG_CHANGE_TOGGLE_D2 <= CONFIG_CHANGE_TOGGLE_D;
		if(CONFIG_CHANGE_TOGGLE_D /= CONFIG_CHANGE_TOGGLE_D2) then
			CONFIG_CHANGE_PULSE <= '1';
		else
			CONFIG_CHANGE_PULSE <= '0';
		end if;
	end if;
end process;


end behavioral;